@@ -5,7 +5,7 @@ use LWP;
 use XML::Simple;
 use strict;
 
-our $VERSION = 'v2.32';
+our $VERSION = 'v2.34';
 
 # ADMIN OPTS - http://www.vmware.com/support/vcd/doc/rest-api-doc-1.5-html/landing-admin_operations.html
 # USER OPTS - http://www.vmware.com/support/vcd/doc/rest-api-doc-1.5-html/landing-user_operations.html
@@ -175,8 +175,12 @@ sub _fault {
   my $message = "\nERROR: ";
   
   if ( length(@error) and ref $error[0] eq 'HTTP::Response' ) {
-    $message .= $error[0]->status_line;    
-    $self->_debug( Dumper(\@error) );
+    $message .= $error[0]->status_line;
+    if ( $error[0]->content ) {
+      $self->_debug(Dumper(\@error));
+      my $ret = $self->_parse_xml($error[0]->content);
+      $message .= ' : '. $ret->{message};
+    }
     die $message;
   }
   
@@ -208,13 +212,19 @@ sub _xml_response {
   $self->_debug_with_level(3,"Received XML Content: \n\n" . $response->content . "\n\n");
   if ( $response->is_success ) {
     return undef unless $response->content;
-    my $data = XMLin( $response->content, ForceArray => 1 );
-    return $data;
+    return $self->_parse_xml( $response->content );
   } else {
     $self->_fault($response);
   }
 }
 
+sub _parse_xml {
+  my $self = shift @_;
+  my $xml  = shift @_;
+  my $data = XMLin( $xml, ForceArray => 1 );
+  return $data;
+}
+
 =head1 REST METHODS
 
 These are direct access to the REST web methods.
@@ -1302,6 +1312,145 @@ sub vdc_list {
   return $self->_xml_response($response);
 }
 
+=head2 vapp_create_from_template($url,$name,$netid,$fencemode,$template_href,$IpAddressAllocationMode,$vcdid,$tmplid)
+
+Create a vapp from a vapp template.
+
+Given a name, VDC, template and network, instantiate the vapp template with the given
+settings and other defaults.
+
+* Fencemode can be: bridged, isolated, or natRouted
+* IP Allocation mode can be: NONE, MANUAL, POOL, DHCP
+
+An array(ref) is returned. The first element is the task href, if one was
+created. The second element is the HTTP reqest object returned by the server.
+
+=cut
+
+# http://pubs.vmware.com/vcd-51/index.jsp?topic=%2Fcom.vmware.vcloud.api.reference.doc_51%2Fdoc%2Ftypes%2FInstantiateVAppTemplateParamsType.html
+
+sub vapp_create_from_template {
+  my $self = shift @_;
+  my $url  = shift @_;
+  
+  my $name = shift @_;
+  my $netid = shift @_;
+  my $fencemode = shift @_;
+  my $template_href = shift @_;
+  my $IpAddressAllocationMode = shift @_;
+
+  my $vdcid  = shift @_;  
+  my $tmplid = shift @_;
+
+  $self->_debug("API: vapp_create($url)\n") if $self->{debug};
+
+  # XML to build
+
+  my $xml = '<InstantiateVAppTemplateParams name="'.$name.'" xmlns="http://www.vmware.com/vcloud/v1.5" xmlns:ovf="http://schemas.dmtf.org/ovf/envelope/1" >
+	<Description>Example FTP Server vApp</Description>
+	<InstantiationParams>
+		<NetworkConfigSection>
+			<ovf:Info>Configuration parameters for vAppNetwork</ovf:Info>
+			<NetworkConfig networkName="vAppNetwork">
+				<Configuration>
+					<ParentNetwork href="'.$netid.'"/>
+					<FenceMode>'.$fencemode.'</FenceMode>
+				</Configuration>
+			</NetworkConfig>
+		</NetworkConfigSection>
+	</InstantiationParams>
+	<Source href="'.$template_href.'"/>
+  </InstantiateVAppTemplateParams>';
+
+  my $ret = $self->post($url,'application/vnd.vmware.vcloud.instantiateVAppTemplateParams+xml',$xml);
+  my $task_href = $ret->[2]->{Tasks}->[0]->{Task}->{task}->{href};
+  return wantarray ? ( $task_href, $ret ) : \( $task_href, $ret );
+}
+
+=head2 vapp_create_from_sources($url,$name,$netid,$fencemode,$template_href,$IpAddressAllocationMode,$vcdid,$sourcesref)
+
+Given one or more source HREFs, such as VMs withing in an existing template or 
+a VM catalog item, create a vApp.
+
+Details of the create task will be returned.
+
+* Fencemode can be: bridged, isolated, or natRouted
+* IP Allocation mode can be: NONE, MANUAL, POOL, DHCP
+
+=cut
+
+# http://pubs.vmware.com/vcd-51/topic/com.vmware.vcloud.api.doc_51/GUID-9E04772F-2BA9-42A9-947D-4EE7A05A6EE0.html
+
+sub vapp_create_from_sources {
+  my $self = shift @_;
+  my $url  = shift @_;
+  
+  my $name = shift @_;
+  my $netid = shift @_;
+  my $fencemode = shift @_;
+  my $template_href = shift @_;
+  my $IpAddressAllocationMode = shift @_;
+
+  my $vdcid  = shift @_;  
+  my $tmplid = shift @_;
+
+  $self->_debug("API: vapp_create($url)\n") if $self->{debug};
+
+  # XML to build
+
+my $xml = '<ComposeVAppParams name="'.$name.'" xmlns="http://www.vmware.com/vcloud/v1" xmlns:ovf="http://schemas.dmtf.org/ovf/envelope/1">
+  <InstantiationParams>
+    <NetworkConfigSection>
+      <ovf:Info>Configuration parameters for logical networks</ovf:Info>
+      <NetworkConfig networkName="'.$netid.'">
+        <Configuration>
+          <ParentNetwork href="'.$netid.'"/> 
+          <FenceMode>'.$fencemode.'</FenceMode>
+        </Configuration>
+      </NetworkConfig>
+    </NetworkConfigSection>
+  </InstantiationParams>
+  <Item>
+    <Source href="'.$template_href.'"/>
+    <InstantiationParams>
+      <NetworkConnectionSection
+        type="application/vnd.vmware.vcloud.networkConnectionSection+xml"
+        href="'.$template_href.'/networkConnectionSection/" ovf:required="false">
+        <ovf:Info/>
+        <PrimaryNetworkConnectionIndex>0</PrimaryNetworkConnectionIndex>
+        <NetworkConnection network="'.$netid.'">
+          <NetworkConnectionIndex>0</NetworkConnectionIndex>
+          <IsConnected>true</IsConnected>
+          <IpAddressAllocationMode>'.$IpAddressAllocationMode.'</IpAddressAllocationMode>
+        </NetworkConnection>
+      </NetworkConnectionSection>
+    </InstantiationParams>
+
+  </Item>
+  <AllEULAsAccepted>true</AllEULAsAccepted>
+</ComposeVAppParams>';
+
+my $xml = '
+<InstantiateVAppTemplateParams name="'.$name.'" xmlns="http://www.vmware.com/vcloud/v1.5" xmlns:ovf="http://schemas.dmtf.org/ovf/envelope/1" >
+	<Description>Example FTP Server vApp</Description>
+	<InstantiationParams>
+		<NetworkConfigSection>
+			<ovf:Info>Configuration parameters for vAppNetwork</ovf:Info>
+			<NetworkConfig networkName="vAppNetwork">
+				<Configuration>
+					<ParentNetwork href="'.$netid.'"/>
+					<FenceMode>'.$fencemode.'</FenceMode>
+				</Configuration>
+			</NetworkConfig>
+		</NetworkConfigSection>
+	</InstantiationParams>
+	<Source href="'.$template_href.'"/>
+</InstantiateVAppTemplateParams>
+';
+
+  return $self->post($url,'application/vnd.vmware.vcloud.instantiateVAppTemplateParams+xml',$xml);
+}
+
 =head2 vapp_get($vappid or $vapp_href)
 
 As a parameter, this method thakes the raw numeric id of the vApp or the full URL.
@@ -1394,7 +1543,7 @@ dearly love a few changes, that might help things:
 
 =head1 VERSION
 
-  Version: v2.32 (2013-03-29)
+  Version: v2.34 (2013-04-09)
 
 =head1 AUTHOR
 
