@@ -6,7 +6,7 @@ use VMware::API::vCloud;
 use VMware::vCloud::vApp;
 use strict;
 
-our $VERSION = 'v2.32';
+our $VERSION = 'v2.34';
 
 =head1 NAME
 
@@ -148,6 +148,9 @@ Details of the create task will be returned.
 
 =cut
 
+# bridged, isolated, or natRouted
+# NONE, MANUAL, POOL, DHCP
+
 sub create_vapp_from_template {
   my $self = shift @_;
   my $name = shift @_;
@@ -170,85 +173,58 @@ sub create_vapp_from_template {
   my $fencemode = 'bridged'; # bridged, isolated, or natRouted
   my $IpAddressAllocationMode = 'POOL'; # NONE, MANUAL, POOL, DHCP
 
-  # XML to build
-
-my $xml = '<ComposeVAppParams name="'.$name.'" xmlns="http://www.vmware.com/vcloud/v1" xmlns:ovf="http://schemas.dmtf.org/ovf/envelope/1">
-  <InstantiationParams>
-    <NetworkConfigSection>
-      <ovf:Info>Configuration parameters for logical networks</ovf:Info>
-      <NetworkConfig networkName="'.$netid.'">
-        <Configuration>
-          <ParentNetwork href="'.$netid.'"/> 
-          <FenceMode>'.$fencemode.'</FenceMode>
-        </Configuration>
-      </NetworkConfig>
-    </NetworkConfigSection>
-  </InstantiationParams>
-  <Item>
-    <Source href="'.$template{href}.'"/>
-    <InstantiationParams>
-      <NetworkConnectionSection
-        type="application/vnd.vmware.vcloud.networkConnectionSection+xml"
-        href="'.$template{href}.'/networkConnectionSection/" ovf:required="false">
-        <ovf:Info/>
-        <PrimaryNetworkConnectionIndex>0</PrimaryNetworkConnectionIndex>
-        <NetworkConnection network="'.$netid.'">
-          <NetworkConnectionIndex>0</NetworkConnectionIndex>
-          <IsConnected>true</IsConnected>
-          <IpAddressAllocationMode>'.$IpAddressAllocationMode.'</IpAddressAllocationMode>
-        </NetworkConnection>
-      </NetworkConnectionSection>
-    </InstantiationParams>
-
-  </Item>
-  <AllEULAsAccepted>true</AllEULAsAccepted>
-</ComposeVAppParams>';
-
-
-#  <Item>
-#    <Source href="http://vcloud.example.com/api/v1.0/vApp/vm-4"/>
-#    <InstantiationParams>
-#      <NetworkConnectionSection
-#        type="application/vnd.vmware.vcloud.networkConnectionSection+xml"
-#        href="http://vcloud.example.com/api/v1.0/vApp/vm-4/
-#        networkConnectionSection/" ovf:required="false">
-#        <ovf:Info/>
-#        <PrimaryNetworkConnectionIndex>0</PrimaryNetworkConnectionIndex>
-#        <NetworkConnection network="CRMApplianceNetwork">
-#          <NetworkConnectionIndex>0</NetworkConnectionIndex>
-#          <IsConnected>true</IsConnected>
-#          <IpAddressAllocationMode>DHCP</IpAddressAllocationMode>
-#        </NetworkConnection>
-#      </NetworkConnectionSection>
-#    </InstantiationParams>
-#  </Item>
-#  <Item>
-#    <Source href="http://vcloud.example.com/api/v1.0/vAppTemplate/vappTemplate-114"/>
-#  </Item>
-
-#my $ret = $self->{api}->post($url,'application/vnd.vmware.vcloud.composeVAppParams+xml',$xml);
-
-my $xml = '
-<InstantiateVAppTemplateParams name="'.$name.'" xmlns="http://www.vmware.com/vcloud/v1" xmlns:ovf="http://schemas.dmtf.org/ovf/envelope/1" >
-	<Description>Example FTP Server vApp</Description>
-	<InstantiationParams>
-		<NetworkConfigSection>
-			<ovf:Info>Configuration parameters for vAppNetwork</ovf:Info>
-			<NetworkConfig networkName="vAppNetwork">
-				<Configuration>
-					<ParentNetwork href="'.$netid.'"/>
-					<FenceMode>'.$fencemode.'</FenceMode>
-				</Configuration>
-			</NetworkConfig>
-		</NetworkConfigSection>
-	</InstantiationParams>
-	<Source href="'.$template{href}.'"/>
-</InstantiateVAppTemplateParams>
-';
-
-  return $self->{api}->post($url,'application/vnd.vmware.vcloud.instantiateVAppTemplateParams+xml',$xml);
+  return $self->{api}->vapp_create_from_template($url,$name,$netid,'bridged',$template{href},'POOL',$vdcid,$tmplid);
 }
 
+=head2 create_vapp_from_sources(...)
+
+Create a vApp from varied sources
+
+Details of the create task will be returned.
+
+=cut
+
+# bridged, isolated, or natRouted
+# NONE, MANUAL, POOL, DHCP
+
+sub create_vapp_from_sources {
+  my $self = shift @_;
+  my $name = shift @_;
+
+  my $vdcid  = shift @_;  
+  my $tmplid = shift @_;
+  my $netid  = shift @_;
+  
+  my %template = $self->get_template($tmplid);
+  my %vdc = $self->get_vdc($vdcid);
+
+  my @links = @{$vdc{Link}};
+  my $url;
+
+  for my $ref (@links) {
+    #$url = $ref->{href} if $ref->{type} eq 'application/vnd.vmware.vcloud.composeVAppParams+xml';
+    $url = $ref->{href} if $ref->{type} eq 'application/vnd.vmware.vcloud.instantiateVAppTemplateParams+xml';
+  }
+
+  my $fencemode = 'bridged'; # bridged, isolated, or natRouted
+  my $IpAddressAllocationMode = 'POOL'; # NONE, MANUAL, POOL, DHCP
+
+  return $self->{api}->vapp_create_from_sources($url,$name,$netid,'bridged',$template{href},'POOL',$vdcid,$tmplid);
+}
+=head2 delete_vapp($vapp_href)
+
+Given the org HREF, call a delete on it.
+
+=cut
+
+sub delete_vapp {
+  my $self = shift @_;
+  my $href = shift @_;
+  $self->purge(); # Clear cache when deleting
+  return $self->{api}->delete($href);
+}
+
+
 =head2 get_vapp($vappid)
 
 Given an vApp id, it returns a vApp object for that vApp.
@@ -414,6 +390,7 @@ Given the org HREF, call a delete on it.
 sub delete_catalog {
   my $self = shift @_;
   my $href = shift @_;
+  $self->purge(); # Clear cache when deleting  
   return $self->{api}->delete($href);
 }
 
@@ -438,6 +415,7 @@ Given the org HREF, call a delete on it.
 sub delete_org {
   my $self = shift @_;
   my $href = shift @_;
+  $self->purge(); # Clear cache when deleting  
   return $self->{api}->delete($href);
 }
 
@@ -450,6 +428,7 @@ Given the org network HREF, call a delete on it.
 sub delete_org_network {
   my $self = shift @_;
   my $href = shift @_;
+  $self->purge(); # Clear cache when deleting  
   return $self->{api}->delete($href);
 }
 
@@ -547,16 +526,17 @@ sub list_orgs {
   my $self = shift @_;
   my $orgs = our $cache->get('list_orgs:');
 
-  #unless ( defined $orgs ) {
+  unless ( defined $orgs ) {
     $orgs = {};
     my $ret = $self->{api}->org_list();
 
     for my $orgname ( keys %{$ret->{Org}} ) {
       warn "Org type of $ret->{Org}->{$orgname}->{type} listed for $orgname\n" unless $ret->{Org}->{$orgname}->{type} eq 'application/vnd.vmware.vcloud.org+xml';
-      $orgs->{$orgname} = $ret->{Org}->{$orgname}->{href};
+      my $href = $ret->{Org}->{$orgname}->{href};
+      $orgs->{$orgname} = $href;
     }
     $cache->set('list_orgs:',$orgs); 
-  #}
+  }
   
   return wantarray ? %$orgs : $orgs if defined $orgs;
 }
@@ -583,6 +563,7 @@ Given the org VDC HREF, call a delete on it.
 sub delete_vdc {
   my $self = shift @_;
   my $href = shift @_;
+  $self->purge(); # Clear cache when deleting
   return $self->{api}->delete($href);
 }
 
@@ -1025,7 +1006,7 @@ identifier of an object. This module implements this best practice.
 
 =head1 VERSION
 
-  Version: v2.32 (2013-03-29)
+  Version: v2.34 (2013-04-09)
 
 =head1 AUTHOR
 
@@ -1048,4 +1029,4 @@ and other such items.
 
   Released under Perl Artistic License
 
-=cut
+=cut
\ No newline at end of file
